function price = CIR_zero_coupon(r0, k, theta, sigma, T)
% --------------------------------------------------------------------------------------------------
% Calculate zero-coupon bond price in CIR model. Formula taken from "Dynamic Asset Pricing Theory",
% page 160, Duffie (2001), and http://www.quantlet.com/mdstat/scripts/sfe/html/sfenode51.html
% --------------------------------------------------------------------------------------------------
% r0                        ... initial interest rate
% [k, theta, sigma]         ... parameters of CIR process
% T                         ... maturity of bond
% --------------------------------------------------------------------------------------------------
% sample call: CIR_zero_coupon(0.01, 0.25, 0.02, 0.1, 5)
% --------------------------------------------------------------------------------------------------

gamma = sqrt(k^2+2*sigma^2);
tmp = exp(gamma*T);
tmp2 = (gamma+k)*(tmp-1) + 2*gamma;
beta = 2*(1-tmp) / tmp2
alpha = 2*k*theta/sigma^2 * (log(2*gamma*exp((gamma+k)/2*T)) - log(tmp2))
price = exp(alpha + beta*r0);